# coding: utf-8
"""
spec pocha reporter
"""

import time

from colored import fg, attr

from pocha.reporters.base import Reporter
from pocha.util import print_failures


_pass = fg('green')
_fail = fg('red')
_skip = fg('cyan')
_dim = attr('dim')
_reset = attr('reset')

class DotReporter(Reporter):

    def __init__(self):
        self.start = None
        self.passing = 0
        self.failing = 0
        self.skipping = 0
        self.total = 0
        self.failures = []

    def beforeTests(self, stdout):
        self.start = time.time()*1000
        stdout.write('\n')

    def afterTests(self, stdout):
        duration = time.time()*1000 - self.start

        if self.total > 0:
            stdout.write('\n\n')

        if stdout.isatty():
            stdout.write('  %s%d passing%s %s(%dms)%s\n' %
                         (_pass, self.passing, _reset, _dim, duration, _reset))

        else:
            stdout.write('  %d passing (%dms)\n' % (self.passing, duration))

        if self.skipping > 0:
            if stdout.isatty():
                stdout.write('  %s%d pending%s\n' % (_skip, self.skipping, _reset))

            else:
                stdout.write('  %d pending\n' % self.skipping)

        if self.failing != 0:
            if stdout.isatty():
                stdout.write('  %s%d failing%s\n' % (_fail, self.failing, _reset))

            else:
                stdout.write('  %d failing\n' % self.failing)

            stdout.write('\n')
            # list the tests in order with their associated stacktrace
            print_failures(self.failures, stdout)

        stdout.write('\n')

    def afterTest(self, stdout, test):

        if self.total == 0:
            stdout.write('  ')

        self.total += 1

        if test.status == 'pass':
            if stdout.isatty():
                stdout.write('%s.%s' % (_dim, _reset))

            else:
                stdout.write('.')

            self.passing += 1

        elif test.status == 'fail':
            if stdout.isatty():
                stdout.write('%s.%s' % (_fail, _reset))

            else:
                stdout.write('.')

            self.failures.append((test.name, test.exc_info))
            self.failing += 1

        elif test.status == 'skip':
            if stdout.isatty():
                stdout.write('%s.%s' % (_skip, _reset))

            else:
                stdout.write('.')

            self.skipping += 1

        stdout.flush()



"""
unit testing utilities
"""

from lxml import etree

from robber.matchers.base import Base
from robber import expect

def init():
    class XPath(Base):

        def matches(self):
            root = etree.fromstring(self.actual)
            return root.xpath(self.expected) is not None

        def failure_message(self):
            return 'Expected "%s" to match xpath "%s"' % (self.actual, self.expected)

    expect.register('xpath', XPath)



# coding: utf-8

import sh

from robber import expect
from pocha import describe, it

import util
util.init()


@describe('@before and @after tests')
def pocha_cli():

    @it('fails and stops running as soon as the first @before fails within a few @its')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/multiple_its_with_before_failure.py',
                        _ok_code=[1],
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  1\) "before all" hook

  0 passing \(\d+ms\)
  1 failing

  1\) "before all" hook:
     Exception: doing it on purpose
     File "test/input/multiple_its_with_before_failure.py", line 6, in setup
       raise Exception\('doing it on purpose'\)

''')

    @it('fails when @after fails within a few @its')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/multiple_its_with_after_failure.py',
                        _ok_code=[1],
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  ✓ can run a passing it
  ✓ can run another passing it
  1\) "after all" hook

  2 passing \(\d+ms\)
  1 failing

  1\) "after all" hook:
     Exception: doing it on purpose
     File "test/input/multiple_its_with_after_failure.py", line 6, in teardown
       raise Exception\('doing it on purpose'\)

''')

    @it('fails and stops running as soon as the first @before fails within a @describe')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/multiple_its_in_a_describe_with_before_failure.py',
                        _ok_code=[1],
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  a describe
    1\) "before all" hook

  0 passing \(\d+ms\)
  1 failing

  1\) "before all" hook:
     Exception: doing it on purpose
     File "test/input/multiple_its_in_a_describe_with_before_failure.py", line 8, in teardown
       raise Exception\('doing it on purpose'\)

''')

    @it('fails when @after fails within a @describe')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/multiple_its_in_a_describe_with_after_failure.py',
                        _ok_code=[1],
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  a describe
    ✓ can run a passing it
    ✓ can run another passing it
  1\) "after all" hook

  2 passing \(\d+ms\)
  1 failing

  1\) "after all" hook:
     Exception: doing it on purpose
     File "test/input/multiple_its_in_a_describe_with_after_failure.py", line 8, in teardown
       raise Exception\('doing it on purpose'\)

''')



"""
pocha test discovery module responsible for creating a dictionary containing
the testing hierarchy as represented by the underlying tests.

"""
import os
import imp
import sys

from collections import OrderedDict

from pocha import common


def __load_modules(path):
    modules = []

    if os.path.isfile(path) and path.endswith('.py'):
        modules.append(path)

    elif os.path.isdir(path):
        # we do recursively attempt to find tests in a directory that contains
        # the pocha.ignore file in it
        if os.path.exists(os.path.join(path, 'pocha.ignore')):
            return modules

        for filename in os.listdir(path):
            modules += __load_modules(os.path.join(path, filename))

    return modules

class FalseyDict(dict):

    def __init__(self, dictionary):
        self.dict = dictionary

    def __getitem__(self, key):

        if key in self.dict.keys():
            return self.dict[key]

        else:
            # by returning False the evaluation can happen for tags that
            # are not defined
            return False


def filter_tests(tests, expression):
    filtered_tests = OrderedDict()

    for (key, thing) in tests.items():
        if thing.only:
            return OrderedDict({
                thing.name: thing
            })

        if thing.type == 'test':
            if expression is None:
                filtered_tests[key] = thing
                continue

            global_tags = FalseyDict(thing.tags)

            if eval(expression, global_tags):
                filtered_tests[key] = thing

        elif thing.type == 'suite':
            thing.tests = filter_tests(thing.tests, expression)

            if len(thing.tests) != 0:
                filtered_tests[key] = thing

    return filtered_tests


def search(path, expression):
    modules = __load_modules(path)

    # load each module and then we'll have a complete list of the tests
    # to run
    for module in modules:
        module_path = os.path.dirname(module)
        sys.path.insert(0, '.')
        sys.path.insert(0, module_path)
        try:
            name = module.replace('/', '.')
            if '.py' in name:
                name = name[:name.index('.py')]
            __import__(name)
        finally:
            sys.path.remove('.')
            sys.path.remove(module_path)

    return filter_tests(common.TESTS, expression)



# coding: utf-8

import sh

from robber import expect

from pocha import describe, it

import util
util.init()


@describe('filtering tagged tests')
def pocha_cli():

    @it('can run an it filtered on a unique tag')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/tagged_its.py',
                        '--filter', 'third',
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  ✓ third it

  1 passing \(\d+ms\)

''')

    @it('can run a set of its filtered with the same tag')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/tagged_its.py',
                        '--filter', 'edge',
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  ✓ first it
  ✓ third it

  2 passing \(\d+ms\)

''')

    @it('can run with `or` expression')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/tagged_its.py',
                        '--filter', 'first or second',
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  ✓ first it
  ✓ second it

  2 passing \(\d+ms\)

''')

    @it('can run with `and` expressions')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/tagged_its.py',
                        '--filter', 'first and edge',
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  ✓ first it

  1 passing \(\d+ms\)

''')

    @it('can run with `not` expression')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/tagged_its.py',
                        '--filter', 'not middle',
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  ✓ first it
  ✓ third it

  2 passing \(\d+ms\)

''')



from pocha import it


@it('first it', tags=['first', 'edge'])
def _():
    pass

@it('second it', tags=['second', 'middle'])
def _():
    pass

@it('third it', tags=['third', 'edge'], only=True)
def _():
    pass



"""
reporters register module
"""

from pocha.reporters.dot import DotReporter
from pocha.reporters.spec import SpecReporter
from pocha.reporters.xunit import XUnitReporter

_REPORTERS = {
    'dot': DotReporter,
    'spec': SpecReporter,
    'xunit': XUnitReporter
}

def get_reporter(name):

    if name not in _REPORTERS.keys():
        raise Exception('reporter "%s" not found' % name)

    return _REPORTERS[name]



from pocha import it


@it('can run a passing it')
def _():
    pass

@it('can run a failing it')
def _():
    raise Exception('failing on purpose')

@it('can run another passing it')
def _():
    pass



# coding: utf-8

import sh

from robber import expect
from pocha import describe, it

import util
util.init()


@describe('@describe tests')
def pocha_cli():

    @it('can run test with local module import')
    def local_module_import():
            cmd = sh.python('pocha/cli.py',
                            'test/input/describe_with_local_module_import.py',
                            _ok_code=[0],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  top level describe
    ✓ can call out to a local module import

  1 passing \(.*ms\)

''')

    @it('can run test with local module from X import Y')
    def local_module_import():
            cmd = sh.python('pocha/cli.py',
                            'test/input/describe_with_local_module_from_import.py',
                            _ok_code=[0],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  top level describe
    ✓ can call out to a local module from import

  1 passing \(.*ms\)

''')

    @it('can run test with local module relative from .. import Y')
    def local_module_import():
            cmd = sh.python('pocha/cli.py',
                            'test/input/describe_with_local_module_relative_from_import.py',
                            _ok_code=[0],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  top level describe
    ✓ can call out to a local module from import

  1 passing \(.*ms\)

''')


    @describe('spec reporter', tags=['spec'])
    def spec():
        @it('can run an empty @describe')
        def _():
            cmd = sh.python('pocha/cli.py',
                            'test/input/describe_with_no_it.py',
                            _ok_code=[0],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''

  0 passing \(\d+ms\)

''')

        @it('can run a @describe with a single passing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            'test/input/describe_with_single_passing_it.py',
                            _ok_code=[0],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  top level describe
    ✓ can run a single passing it

  1 passing \(\d+ms\)

''')

        @it('can run a @describe with multiple passing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            'test/input/describe_with_multiple_passing_it.py',
                            _ok_code=[0],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  top level describe
    ✓ can run a passing it
    ✓ can run another passing it

  2 passing \(\d+ms\)

''')


        @it('can run a @describe with a single failing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            'test/input/describe_with_single_failing_it.py',
                            _ok_code=[1],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  top level describe
    1\) can run a single failing it

  0 passing \(\d+ms\)
  1 failing

  1\) can run a single failing it:
     Exception: failing on purpose
     File "test/input/describe_with_single_failing_it.py", line 9, in failing_it
       raise Exception\('failing on purpose'\)

''')

        @it('can run a @describe with multiple failing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            'test/input/describe_with_multiple_failing_it.py',
                            _ok_code=[1],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  top level describe
    1\) can run a failing it
    2\) can run another failing it

  0 passing \(\d+ms\)
  2 failing

  1\) can run a failing it:
     Exception: failing on purpose
     File "test/input/describe_with_multiple_failing_it.py", line 9, in _
       raise Exception\('failing on purpose'\)

  2\) can run another failing it:
     Exception: failing on purpose again
     File "test/input/describe_with_multiple_failing_it.py", line 13, in _
       raise Exception\('failing on purpose again'\)

''')

    @describe('dot reporter', tags=['dot'])
    def spec():
        @it('can run an empty @describe')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'dot',
                            'test/input/describe_with_no_it.py',
                            _ok_code=[0],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  0 passing \(\d+ms\)

''')

        @it('can run a @describe with a single passing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'dot',
                            'test/input/describe_with_single_passing_it.py',
                            _ok_code=[0],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  .

  1 passing \(\d+ms\)

''')

        @it('can run a @describe with multiple passing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'dot',
                            'test/input/describe_with_multiple_passing_it.py',
                            _ok_code=[0],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  ..

  2 passing \(\d+ms\)

''')


        @it('can run a @describe with a single failing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'dot',
                            'test/input/describe_with_single_failing_it.py',
                            _ok_code=[1],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  .

  0 passing \(\d+ms\)
  1 failing

  1\) can run a single failing it:
     Exception: failing on purpose
     File "test/input/describe_with_single_failing_it.py", line 9, in failing_it
       raise Exception\('failing on purpose'\)

''')

        @it('can run a @describe with multiple failing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'dot',
                            'test/input/describe_with_multiple_failing_it.py',
                            _ok_code=[1],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  ..

  0 passing \(\d+ms\)
  2 failing

  1\) can run a failing it:
     Exception: failing on purpose
     File "test/input/describe_with_multiple_failing_it.py", line 9, in _
       raise Exception\('failing on purpose'\)

  2\) can run another failing it:
     Exception: failing on purpose again
     File "test/input/describe_with_multiple_failing_it.py", line 13, in _
       raise Exception\('failing on purpose again'\)

''')

    @describe('xunit reporter', tags=['xunit'])
    def spec():
        @it('can run an empty @describe')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'xunit',
                            'test/input/describe_with_no_it.py',
                            _ok_code=[0],
                            _tty_out=False)

            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.have.xpath('./testsuite[' +
                                         '@name="Pocha Tests" and ' +
                                         '@tests="0" and ' +
                                         '@errors="0" and ' +
                                         '@failures="0" and ' +
                                         '@skip="0"]')


        @it('can run a @describe with a single passing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'xunit',
                            'test/input/describe_with_single_passing_it.py',
                            _ok_code=[0],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.have.xpath('./testsuite[' +
                                         '@name="Pocha Tests" and ' +
                                         '@tests="1" and ' +
                                         '@errors="0" and ' +
                                         '@failures="0" and ' +
                                         '@skip="0"]')

            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="can run a single passing it" and ' +
                                         '@classname=""]')

        @it('can run a @describe with multiple passing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'xunit',
                            'test/input/describe_with_multiple_passing_it.py',
                            _ok_code=[0],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.have.xpath('./testsuite[' +
                                         '@name="Pocha Tests" and ' +
                                         '@tests="2" and ' +
                                         '@errors="0" and ' +
                                         '@failures="0" and ' +
                                         '@skip="0"]')

            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="can run a passing it" and ' +
                                         '@classname=""]')

            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="can run another passing it" and ' +
                                         '@classname=""]')

        @it('can run a @describe with a single failing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'xunit',
                            'test/input/describe_with_single_failing_it.py',
                            _ok_code=[1],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.have.xpath('./testsuite[' +
                                         '@name="Pocha Tests" and ' +
                                         '@tests="1" and ' +
                                         '@errors="0" and ' +
                                         '@failures="1" and ' +
                                         '@skip="0"]')

            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="can run a single failing it" and ' +
                                         '@classname=""]/error')

        @it('can run a @describe with multiple failing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'xunit',
                            'test/input/describe_with_multiple_failing_it.py',
                            _ok_code=[1],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.have.xpath('./testsuite[' +
                                         '@name="Pocha Tests" and ' +
                                         '@tests="1" and ' +
                                         '@errors="0" and ' +
                                         '@failures="2" and ' +
                                         '@skip="0"]')

            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="can run a failing it" and ' +
                                         '@classname=""]/error')

            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="can run another failing it" and ' +
                                         '@classname=""]/error')



"""
pocha utility module
"""

import traceback
import os


from colored import fg, attr

_fail = fg('red')
_reset = attr('reset')


class EasyDict(dict):

    def __init__(self, dictionary):
        for (key, value) in dictionary.items():
            self.__setitem__(key, value)

    def __getattr__(self, key):
        if key in self:
            return self[key]


def print_failures(failures, stdout):
    """
    pretty print failures in a common way to most reporters
    """
    for index in range(0, len(failures)):
        (name, exc_info) = failures[index]
        # extract the traceback and drop everything after the first part
        # which is in the pocha/runner.py file
        parts = traceback.extract_tb(exc_info[2])[1:]

        # File "../../", line 13, in _
        tb_lines = '\n'.join([
            '  File "%s", line %d, in %s\n    %s\n' %
            (os.path.relpath(path), line, method, exception)
            for path, line, method, exception in parts
        ])

        tb_lines = ['   %s' % line
                    for line in tb_lines.split('\n') if line.strip() != '']
        stdout.write('  %d) %s:\n' % (index + 1, name))
        exception_name = exc_info[1].__class__.__name__
        if stdout.isatty():
            stdout.write('     %s%s: %s%s\n' %
                         (_fail, exception_name, exc_info[1], _reset))

        else:
            stdout.write('     %s: %s\n' % (exception_name, exc_info[1]))
        stdout.write('\n'.join(tb_lines))
        stdout.write('\n\n')



from pocha import describe, it


@describe('top level describe')
def describe1():

    @it('can run a failing it')
    def _():
        raise Exception('failing on purpose')

    @it('can run another failing it')
    def _():
        raise Exception('failing on purpose again')



# coding: utf-8

import sh

from robber import expect
from pocha import describe, it


@describe('cli tests')
def cli_tests():

    @it('fails when provided an inexistent path')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'inexistent_path',
                        _ok_code=[2],
                        _tty_out=False)
        stderr = cmd.stderr.decode('utf-8')
        expect(stderr).to.eq('''Usage: cli.py [OPTIONS] [PATH]

Error: Invalid value for path: "inexistent_path" not found
''')
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.eq('')


    @it('fails when provided an invalid reporter')
    def _():
        cmd = sh.python('pocha/cli.py',
                        '--reporter', 'bananas',
                        _ok_code=[2],
                        _tty_out=False)
        stderr = cmd.stderr.decode('utf-8')
        expect(stderr).to.eq('''Usage: cli.py [OPTIONS] [PATH]

Error: Invalid value for "--reporter" / "-r": invalid choice: bananas. (choose from spec, dot, xunit)
''')
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.eq('')

    @it('can display the help menu')
    def _():
        cmd = sh.python('pocha/cli.py',
                        '--help',
                        _ok_code=[0],
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.contain('''Usage: cli.py [OPTIONS] [PATH]

Options:
  -r, --reporter [spec|dot|xunit]
  -f, --filter TEXT
  --version                       Show the version and exit.
  --help                          Show this message and exit.
''')
        stderr = cmd.stderr.decode('utf-8')
        expect(stderr).to.eq('')

    @it('can run a test suite')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/describe_with_multiple_passing_it.py',
                        _ok_code=[0],
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.eq(u'''
  top level describe
    ✓ can run a passing it
    ✓ can run another passing it

  2 passing (0ms)

''')
        stderr = cmd.stderr.decode('utf-8')
        expect(stderr).to.eq('')



from pocha import before_each, it


@before_each
def setup():
    raise Exception('doing it on purpose')

@it('can run a passing it')
def _():
    pass

@it('can run another passing it')
def _():
    pass



#!env python
"""
cli entry point for pocha
"""

import os
import sys

import click

from pocha import discover
from pocha import runner

from pocha.reporters.registry import get_reporter

@click.version_option(prog_name='pocha', version='0.6.1')
@click.command()
@click.argument('path', default='test')
@click.option('--reporter', '-r',
              default='spec',
              type=click.Choice(['spec','dot','xunit']))
@click.option('--filter', '-f', 'expression', default=None)
def cli(path, reporter, expression):

    if os.path.exists(path):
        tests = discover.search(path, expression)
        reporter_object = get_reporter(reporter)()

        if runner.run_tests(tests, reporter_object):
            sys.exit(1)

    else:
        raise click.BadParameter('"%s" not found' % path, param_hint='path')

if __name__ == '__main__':
    cli()



"""
pocha decorator module
"""

from collections import OrderedDict

from pocha.util import EasyDict

TESTS = OrderedDict()

SUITE_STACK = [EasyDict({
    'name': '__default__',
    'type': 'suite',
    'tests': OrderedDict(),
    'skip': False,
    'only': False,
    'tags': {},

    'before': [],
    'after': [],
    'before_each': [],
    'after_each': []
})]

# __default__ suite is created but never recorded as a suite
TESTS['__default__'] = SUITE_STACK[0]


def handle_tags(tags):

    if tags is None:
        return {}

    if isinstance(tags, dict):
        return tags

    if isinstance(tags, list):
        return {key: True for key in tags}

    raise Exception('unexpected tags type %s' % type(tags))


class describe(object):
    """
    decorator used to define a test suite by name

    params:
        name - name of the test suite
        tags - list of tags used to tag this test suite. You can provide a list
               of tags ie `tags=['foo', 'bar']` or you can provide a dictionary
               of tags ie `tags={ 'foo': 1, 'bar': True }`
        only - when set to True, pocha will only run this test or suite and skip
               all others without producing a skipped test result
        skip - when set to True, pocha will skip this test and produce a skipped
               test result
    """

    def __init__(self,
                 name,
                 only=False,
                 skip=False,
                 tags=None):
        self.name = name
        self.only = only
        self.skip = skip
        self.tags = handle_tags(tags)

    def __call__(self, func):
        suite = EasyDict({
            'name': self.name,
            'type': 'suite',
            'tests': OrderedDict(),
            'only': self.only,
            'skip': self.skip,
            'tags': self.tags,

            'before': [],
            'after': [],
            'before_each': [],
            'after_each': []
        })
        SUITE_STACK[-1].tests[self.name] = suite
        SUITE_STACK.append(suite)

        # run the underlying describe function which in turns executes any other
        # @describe and @it decorators
        func()
        SUITE_STACK.pop()
        return func


class it(object):
    """
    decorator used to define a test case by name

    params:
        name - name of the test case
        tags - list of tags used to tag this test. You can provide a list of
               tags ie `tags=['foo', 'bar']` or you can provide a dictionary
               of tags ie `tags={ 'foo': 1, 'bar': True }`
        only - when set to True, pocha will only run this test or suite and skip
               all others without producing a skipped test result
        skip - when set to True, pocha will skip this test and produce a skipped
               test result
    """

    def __init__(self,
                 name,
                 only=False,
                 skip=False,
                 tags=None):
        self.name = name
        self.only = only
        self.skip = skip
        self.tags = handle_tags(tags)

    def __call__(self, func):
        current_stack = SUITE_STACK[-1]
        tags = current_stack.tags.copy()
        tags.update(self.tags)
        current_stack.tests[self.name] = EasyDict({
            'name': self.name,
            'type': 'test',
            'func': func,
            'tags': tags,
            'only': self.only,
            'skip': self.skip or current_stack.skip
        })
        func.name = self.name
        return func


def before(func):
    """
    decorator used to define a function to execute before the current suite is
    executed
    """
    current_stack = SUITE_STACK[-1]
    current_stack.before.append(func)
    return func


def after(func):
    """
    decorator used to define a function to execute after the current suite is
    executed
    """
    current_stack = SUITE_STACK[-1]
    current_stack.after.append(func)
    return func


def before_each(func):
    """
    decorator used to define a function to execute before each test case
    """
    current_stack = SUITE_STACK[-1]
    current_stack.before_each.append(func)
    return func


def after_each(func):
    """
    decorator used to define a function to execute after each test case
    """
    current_stack = SUITE_STACK[-1]
    current_stack.after_each.append(func)
    return func


# aliases
beforeEach = before_each
afterEach = after_each



# coding: utf-8
"""
spec pocha reporter
"""

import time

from colored import fg, attr

from pocha.reporters.base import Reporter
from pocha.util import print_failures

_pass = fg('green')
_fail = fg('red')
_skip = fg('cyan')
_dim = attr('dim')
_reset = attr('reset')

class SpecReporter(Reporter):

    def __init__(self):
        self.start = None
        self.depth = 0
        self.suites = 0
        self.passing = 0
        self.failing = 0
        self.skipping = 0
        self.failures = []

    def beforeTests(self, stdout):
        self.start = time.time()*1000
        stdout.write('\n')

    def beforeSuite(self, stdout, suite):
        if self.depth == 0 and self.suites != 0:
            stdout.write('\n')

        self.depth += 2
        self.suites += 1
        stdout.write('%s%s\n' % (' ' * self.depth, suite.name))

    def beforeTest(self, stdout, test):
        pass

    def afterTests(self, stdout):
        duration = time.time()*1000 - self.start
        stdout.write('\n')
        if stdout.isatty():
            stdout.write('  %s%d passing%s %s(%dms)%s\n' %
                         (_pass, self.passing, _reset, _dim, duration, _reset))

        else:
            stdout.write('  %d passing (%dms)\n' % (self.passing, duration))

        if self.skipping > 0:
            if stdout.isatty():
                stdout.write('  %s%d pending%s\n' % (_skip, self.skipping, _reset))

            else:
                stdout.write('  %d pending\n' % self.skipping)

        if self.failing > 0:
            if stdout.isatty():
                stdout.write('  %s%d failing%s\n' % (_fail, self.failing, _reset))

            else:
                stdout.write('  %d failing\n' % self.failing)

            stdout.write('\n')
            # list the tests in order with their associated stacktrace
            print_failures(self.failures, stdout)

        stdout.write('\n')

    def afterSuite(self, stdout, suite):
        self.depth -= 2

    def afterTest(self, stdout, test):
        padding = ' ' * (self.depth + 2)

        if test.status == 'pass':
            if stdout.isatty():
                stdout.write('%s%s✓%s %s%s%s\n' % (padding, _pass, _reset, _dim, test.name, _reset))

            else:
                stdout.write('%s✓ %s\n' % (padding, test.name))

            self.passing += 1

        elif test.status == 'fail':
            which = len(self.failures) + 1

            if stdout.isatty():
                stdout.write('%s%s%d) %s%s%s\n' % (padding, _fail, which, _dim, test.name, _reset))

            else:
                stdout.write('%s%d) %s\n' % (padding, which, test.name))

            self.failures.append((test.name, test.exc_info))
            self.failing += 1

        elif test.status == 'skip':
            if stdout.isatty():
                stdout.write('%s%s- %s%s\n' % (padding, _skip, test.name, _reset))

            else:
                stdout.write('%s- %s\n' % (padding, test.name))

            self.skipping += 1




# coding: utf-8

import sh

from robber import expect

from pocha import describe, it

import util
util.init()


@describe('skip tests')
def pocha_cli():

    @describe('spec reporter', tags=['spec'])
    def _():

        @it('can run handle skips with @its', only=True)
        def _():
            cmd = sh.python('pocha/cli.py',
                            'test/input/skip_its.py',
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  - first it
  ✓ second it
  - third it

  1 passing \(\d+ms\)
  2 pending
''')

    @describe('dot reporter', tags=['spec'])
    def _():

        @it('can run handle skips with @its')
        def _():
            cmd = sh.python('pocha/cli.py',
                            'test/input/skip_its.py',
                            '--reporter', 'dot',
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  ...

  1 passing \(\d+ms\)
  2 pending
''')

    @describe('xunit reporter', tags=['spec'])
    def _():

        @it('can run handle skips with @its')
        def _():
            cmd = sh.python('pocha/cli.py',
                            'test/input/skip_its.py',
                            '--reporter', 'xunit',
                            _tty_out=False)

            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.have.xpath('./testsuite[' +
                                         '@name="Pocha Tests" and ' +
                                         '@tests="3" and ' +
                                         '@errors="0" and ' +
                                         '@failures="0" and ' +
                                         '@skip="2"]')

            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="first it" and ' +
                                         '@classname="" and ' +
                                         '@time="0.000"]/skipped')

            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="second it" and ' +
                                         '@classname="" and ' +
                                         '@time="0.000"]')

            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="third it" and ' +
                                         '@classname="" and ' +
                                         '@time="0.000"]/skipped')



"""
main init module
"""

from pocha import cli
from pocha import discover
from pocha import reporters
from pocha import runner
from pocha import util

from pocha.common import describe, \
                         it, \
                         before, \
                         after, \
                         before_each, \
                         after_each, \
                         beforeEach, \
                         afterEach



from pocha import after, describe, it


@describe('a describe')
def suite():
    @after
    def teardown():
        raise Exception('doing it on purpose')

    @it('can run a passing it')
    def _():
        pass

    @it('can run another passing it')
    def _():
        pass



# coding: utf-8

import sh

from robber import expect
from pocha import describe, it

import util
util.init()


@describe('@before_each and @after_each tests')
def pocha_cli():

    @it('fails on every @it with the @before_each that fails')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/multiple_its_with_before_each_failure.py',
                        _ok_code=[1],
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  1\) "before each" hook for "can run a passing it"
  2\) "before each" hook for "can run another passing it"

  0 passing \(\d+ms\)
  2 failing

  1\) "before each" hook for "can run a passing it":
     Exception: doing it on purpose
     File "test/input/multiple_its_with_before_each_failure.py", line 6, in setup
       raise Exception\('doing it on purpose'\)

  2\) "before each" hook for "can run another passing it":
     Exception: doing it on purpose
     File "test/input/multiple_its_with_before_each_failure.py", line 6, in setup
       raise Exception\('doing it on purpose'\)

''')

    @it('fails on every failing @after_each but the @it succeeds')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/multiple_its_with_after_each_failure.py',
                        _ok_code=[1],
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  ✓ can run a passing it
  1\) "after each" hook for "can run a passing it"
  ✓ can run another passing it
  2\) "after each" hook for "can run another passing it"

  2 passing \(\d+ms\)
  2 failing

  1\) "after each" hook for "can run a passing it":
     Exception: doing it on purpose
     File "test/input/multiple_its_with_after_each_failure.py", line 6, in setup
       raise Exception\('doing it on purpose'\)

  2\) "after each" hook for "can run another passing it":
     Exception: doing it on purpose
     File "test/input/multiple_its_with_after_each_failure.py", line 6, in setup
       raise Exception\('doing it on purpose'\)

''')



"""
base pocha reporter
"""

class Reporter(object):

    def beforeTests(self, stdout):
        pass

    def beforeSuite(self, stdout, suite):
        pass

    def beforeTest(self, stdout, test):
        pass

    def afterTests(self, stdout):
        pass

    def afterSuite(self, stdout, suite):
        pass

    def afterTest(self, stdout, test):
        pass



from pocha import describe, it


@describe('top level describe')
def describe1():

    @it('can run a passing it')
    def _():
        pass
    
    @it('can run another passing it')
    def _():
        pass



from pocha import describe, it

from test.input import util

@describe('top level describe')
def describe1():

    @it('can call out to a local module from import')
    def _():
        assert util.add(1, 1) == 2



from pocha import describe, it


@describe('level 1 describe')
def describe1():

    @it('level 1 it')
    def it1():
        pass

    @describe('level 2 describe')
    def describe2():

        @it('level 2 it')
        def it2():
            pass



"""
setup.py
"""
import os

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

class PochaTestCommand(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        from pocha.cli import cli
        cli()

def load(filename):
    with open(filename, 'r') as input_file:
        return input_file.read().split('\n')

setup(
    name='pocha',
    version='0.13.0',
    author='Rodney Gomes',
    author_email='rodneygomes@gmail.com',
    url='',

    install_requires=load('requirements.txt'),
    tests_require=load('requirements-dev.txt'),

    cmdclass={'test': PochaTestCommand},
    test_suite='test',
    keywords=[''],
    py_modules=['pocha'],
    packages=find_packages(exclude=['tests']),


    entry_points={
        'console_scripts': [
            'pocha=pocha.cli:cli'
        ]
    },

    license='Apache 2.0 License',
    description='',
    long_description='',
)



"""
pocha runner module
"""

import sys

from pocha.util import EasyDict

def run_tests(tests, reporter):
    reporter.beforeTests(sys.stdout)
    hadfailures = __run_tests(None, tests, reporter, sys.stdout)
    reporter.afterTests(sys.stdout)
    return hadfailures


def __run_tests(suite, tests, reporter, stdout):
    hadfailures = False

    for (key, thing) in tests.items():
        if thing.type == 'test':
            try:
                reporter.beforeTest(stdout, EasyDict({
                    'name': thing.func.name
                }))

                try:
                    for before_each in suite.before_each:
                        before_each()

                except Exception as exception:
                    reporter.afterTest(stdout, EasyDict({
                        'name': '"before each" hook for "%s"' % thing.func.name,
                        'status': 'fail',
                        'exc_info': sys.exc_info()
                    }))
                    continue

                if not thing.skip:
                    thing.func()

                    reporter.afterTest(stdout, EasyDict({
                        'name': thing.func.name,
                        'status': 'pass'
                    }))

                else:
                    reporter.afterTest(stdout, EasyDict({
                        'name': thing.func.name,
                        'status': 'skip'
                    }))

                try:
                    for after_each in suite.after_each:
                        after_each()
                except Exception as exception:
                    reporter.afterTest(stdout, EasyDict({
                        'name': '"after each" hook for "%s"' % thing.func.name,
                        'status': 'fail',
                        'exc_info': sys.exc_info()
                    }))

            except Exception as exception:
                reporter.afterTest(stdout, EasyDict({
                    'name': thing.func.name,
                    'status': 'fail',
                    'exc_info': sys.exc_info()
                }))
                hadfailures = True

        elif thing.type == 'suite':
            beforeHookFailed = False

            if thing.name != '__default__':
                reporter.beforeSuite(stdout, EasyDict({
                    'name': key
                }))

            try:
                for before in thing.before:
                    before()
            except Exception as exception:
                reporter.afterTest(stdout, EasyDict({
                    'name': '"before all" hook',
                    'status': 'fail',
                    'exc_info': sys.exc_info()
                }))
                return True

            hadfailures |= __run_tests(thing, thing.tests, reporter, stdout)

            if thing.name != '__default__':
                reporter.afterSuite(stdout, EasyDict({
                    'name': key
                }))

            try:
                for after in thing.after:
                    after()
            except Exception as exception:
                reporter.afterTest(stdout, EasyDict({
                    'name': '"after all" hook',
                    'status': 'fail',
                    'exc_info': sys.exc_info()
                }))
                return True

    return hadfailures



# coding: utf-8

import sh

import util

from robber import expect

from pocha import describe, it


@describe('only tests')
def pocha_cli():

    @it('can run a single @it with only flag')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/only_its.py',
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  ✓ third it

  1 passing \(\d+ms\)

''')

    @it('can not override an only @it with a filter')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/only_its.py',
                        '--filter', 'first',
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  ✓ third it

  1 passing \(\d+ms\)

''')

    @it('can run a single @describe with only flag')
    def _():
        cmd = sh.python('pocha/cli.py',
                        'test/input/only_describe.py',
                        _tty_out=False)
        stdout = cmd.stdout.decode('utf-8')
        expect(stdout).to.match(u'''
  second describe
    ✓ third it
    ✓ fourth it

  2 passing \(\d+ms\)

''')



# coding: utf-8
"""
xunit pocha reporter
"""

import time
import traceback

from xml.etree.ElementTree import Element, SubElement
from xml.etree import ElementTree
from xml.dom import minidom

from pocha.reporters.base import Reporter


# taken from https://gist.github.com/zlalanne/5711847
def CDATA(text=None):
    element = ElementTree.Element('![CDATA[')
    element.text = text
    return element

ElementTree._original_serialize_xml = ElementTree._serialize_xml

def _serialize_xml(write, elem, *args, **kwargs):
    if elem.tag == '![CDATA[':
        write('<%s%s]]>' % (elem.tag, elem.text))
        return

    return ElementTree._original_serialize_xml(write, elem, *args, **kwargs)

ElementTree._serialize_xml = ElementTree._serialize['xml'] = _serialize_xml

# taken from https://pymotw.com/2/xml/etree/ElementTree/create.html
def prettify(elem):
    """
    Return a pretty-printed XML string for the Element.
    """
    rough_string = ElementTree.tostring(elem, 'utf-8')
    reparsed = minidom.parseString(rough_string)
    return reparsed.toprettyxml(indent='  ')

class XUnitReporter(Reporter):

    def __init__(self):
        self.start = None
        self.passing = 0
        self.failing = 0
        self.skipping = 0
        self.total = 0
        self.testsuite = Element('testsuite')
        self.test_start = 0
        self.suite_stack = []

    def afterTests(self, stdout):
        self.testsuite.set('name', 'Pocha Tests')
        self.testsuite.set('tests', str(self.total))
        self.testsuite.set('errors', '0')
        self.testsuite.set('failures', str(self.failing))
        self.testsuite.set('skip', str(self.skipping))
        stdout.write(prettify(self.testsuite))

    def beforeSuite(self, stdout, suite):
        self.suite_stack.append(suite.name)

    def afterSuite(self, stdout, suite):
        self.suite_stack.pop(-1)

    def beforeTest(self, stdout, test):
        self.test_start = time.time()

    def afterTest(self, stdout, test):
        duration = time.time() - self.test_start
        self.total += 1

        testcase = SubElement(self.testsuite, 'testcase', {
            'classname': ' '.join(self.suite_stack),
            'name': test.name,
            'time': '%.3f' % duration
        })

        if test.status == 'pass':
            self.passing += 1

        if test.status == 'skip':
            skipped = SubElement(testcase, 'skipped')
            self.skipping += 1

        elif test.status == 'fail':
            error = SubElement(testcase, 'error')
            error.append(CDATA('\n'.join(traceback.format_exception(*test.exc_info))))
            self.failing += 1

        stdout.flush()



from pocha import describe, it


@describe('first describe')
def describe1():

    @it('first it')
    def _():
        pass

    @it('second it')
    def _():
        pass

@describe('second describe', only=True)
def describe1():

    @it('third it')
    def _():
        pass

    @it('fourth it')
    def _():
        pass



# coding: utf-8

import sh

from robber import expect

from pocha import describe, it

import util
util.init()

@describe('@it tests')
def pocha_cli():

    @describe('spec reporter', tags=['spec'])
    def spec():
        @it('can run a test with a single passing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            'test/input/single_passing_it.py',
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  ✓ can run a single passing it

  1 passing \(\d+ms\)

''')


        @it('can run a test with a single failing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            'test/input/single_failing_it.py',
                            _ok_code=[1],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  1\) can run a single failing it

  0 passing \(\d+ms\)
  1 failing

  1\) can run a single failing it:
     Exception: failing on purpose
     File "test/input/single_failing_it.py", line 6, in failing
       raise Exception\('failing on purpose'\)

''')

        @it('can run a test with mixed @its')
        def _():
            cmd = sh.python('pocha/cli.py',
                            'test/input/mixed_its.py',
                            _ok_code=[1],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  ✓ can run a passing it
  1\) can run a failing it
  ✓ can run another passing it

  2 passing \(\d+ms\)
  1 failing

  1\) can run a failing it:
     Exception: failing on purpose
     File "test/input/mixed_its.py", line 10, in _
       raise Exception\('failing on purpose'\)

''')

    @describe('dot reporter', tags=['dot'])
    def spec():
        @it('can run a test with a single passing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'dot',
                            'test/input/single_passing_it.py',
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  .

  1 passing \(\d+ms\)

''')

        @it('can run a test with a single failing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'dot',
                            'test/input/single_failing_it.py',
                            _ok_code=[1],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  .

  0 passing \(\d+ms\)
  1 failing

  1\) can run a single failing it:
     Exception: failing on purpose
     File "test/input/single_failing_it.py", line 6, in failing
       raise Exception\('failing on purpose'\)

''')

        @it('can run a test with mixed @its')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'dot',
                            'test/input/mixed_its.py',
                            _ok_code=[1],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  ...

  2 passing \(\d+ms\)
  1 failing

  1\) can run a failing it:
     Exception: failing on purpose
     File "test/input/mixed_its.py", line 10, in _
       raise Exception\('failing on purpose'\)

''')

        @it('can run dynamically created @its')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'dot',
                            'test/input/dynamically_created_its.py',
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.match(u'''
  .....

  5 passing \(\d+ms\)

''')

    @describe('xunit reporter', tags=['xunit'])
    def spec():
        @it('can run a test with a single passing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'xunit',
                            'test/input/single_passing_it.py',
                            _tty_out=False)

            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.have.xpath('./testsuite[' +
                                         '@name="Pocha Tests" and ' +
                                         '@tests="1" and ' +
                                         '@errors="0" and ' +
                                         '@failures="0" and ' +
                                         '@skip="0"]')

            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="can run a single passing it" and ' +
                                         '@classname="" and ' +
                                         '@time="0.000"]')


        @it('can run a test with a single failing @it')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'xunit',
                            'test/input/single_failing_it.py',
                            _ok_code=[1],
                            _tty_out=False)

            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.have.xpath('./testsuite[' +
                                         '@name="Pocha Tests" and ' +
                                         '@tests="1" and ' +
                                         '@errors="0" and ' +
                                         '@failures="1" and ' +
                                         '@skip="0"]')

            # XXX: should find a way to validate the stacktrace
            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="can run a single failing it" and ' +
                                         '@classname=""]/error')

        @it('can run a test with mixed @its')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'xunit',
                            'test/input/mixed_its.py',
                            _ok_code=[1],
                            _tty_out=False)
            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.have.xpath('./testsuite[' +
                                         '@name="Pocha Tests" and ' +
                                         '@tests="3" and ' +
                                         '@errors="0" and ' +
                                         '@failures="1" and ' +
                                         '@skip="0"]')

            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="can run a passing it" and ' +
                                         '@classname=""]')

            # XXX: should find a way to validate the stacktrace
            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="can runa a failing it" and ' +
                                         '@classname=""]/error')

            expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                         '@name="can run another passing it" and ' +
                                         '@classname=""]')


        @it('can run dynamically created @its')
        def _():
            cmd = sh.python('pocha/cli.py',
                            '--reporter', 'xunit',
                            'test/input/dynamically_created_its.py',
                            _tty_out=False)

            stdout = cmd.stdout.decode('utf-8')
            expect(stdout).to.have.xpath('./testsuite[' +
                                         '@name="Pocha Tests" and ' +
                                         '@tests="5" and ' +
                                         '@errors="0" and ' +
                                         '@failures="0" and ' +
                                         '@skip="0"]')

            for index in range(1, 6):
                expect(stdout).to.have.xpath('./testsuite/testcase[' +
                                             '@name="passing it #%d" and ' % index+
                                             '@classname=""]')
